<?php
namespace WPUmbrella\Controller\Backup;

if (!defined('ABSPATH')) {
    exit;
}

use WPUmbrella\Core\Models\AbstractController;


class CurrentProcess extends AbstractController
{
	protected function fallbackOldProcess($params){
		$runner = wp_umbrella_get_service('BackupRunner');

        $isRunning = $runner->hasScheduledBatchInProcess();
        $processedDataFile = wp_umbrella_get_service('BackupBatchData')->getData('file');
        $processedDataSql = wp_umbrella_get_service('BackupBatchData')->getData('database');

        if (!$processedDataFile && !$processedDataSql) {
            return $this->returnResponse([
                'is_running' => $isRunning,
                'data' => null
            ]);
        }

        return $this->returnResponse([
            'is_running' => $isRunning,
            'data' => [
                'file' => $processedDataFile ? $processedDataFile->getData() : null,
                'database' => $processedDataSql ? $processedDataSql->getData() : null,
            ]
        ]);
	}
    public function executeGet($params)
    {
		$version = $params['version'] ?? 'old';


		if($version === 'old'){
			return $this->fallbackOldProcess($params);
		}

		if(!defined('WP_UMBRELLA_INIT_BACKUP')){
			define('WP_UMBRELLA_INIT_BACKUP', true);
		}

		$manageProcess = wp_umbrella_get_service('BackupManageProcess');

        $isRunning = $manageProcess->isBackupInProgress();

		$data = null;
		if($isRunning){
			$dataObj = $manageProcess->getBackupData();

		 	$data = $dataObj->getData();
			$data['file'] = $dataObj->getFileData([
				'secure' => true
			]);
			$data['database'] = $dataObj->getDatabaseData([
				'secure' => true
			]);
		}
        return $this->returnResponse([
            'is_running' => $isRunning,
            'data' => $data
        ]);


    }
}
