<?php
namespace WPUmbrella\Controller\Backup;

if (!defined('ABSPATH')) {
    exit;
}

use WPUmbrella\Core\Models\AbstractController;
use WPUmbrella\Core\Exceptions\BackupNotCreated;
use WPUmbrella\Core\Constants\CodeResponse;

class Init extends AbstractController
{
    public function fallbackOldProcess($params)
    {
        $runner = wp_umbrella_get_service('BackupRunner');

        if ($runner->hasScheduledBatchInProcess()) {
            return $this->returnResponse(['code' => 'backup_already_process', 'message' => 'A backup is already in process'], 400);
        }

        try {
            wp_umbrella_get_service('BackupInitProcess')->init($params);
        } catch (BackupNotCreated $e) {
            return $this->returnResponse(['code' => 'error', 'message' => $e->getMessage()], 400);
        }

        return $this->returnResponse(['code' => 'success', 'message' => 'Backup scheduled']);
    }

	/**
	 * @param array $params
	 * 		- string $params['version'] - (old, v1, v2)
	 */
    public function executePost($params)
    {
		$version = $params['version'] ?? 'old';

		if($version === 'old'){
			return $this->fallbackOldProcess($params);
		}

		$manageProcess = wp_umbrella_get_service('BackupManageProcess');

        if ($manageProcess->isBackupInProgress()) {
            return $this->returnResponse(['code' => CodeResponse::BACKUP_ALREADY_PROCESS, 'message' => 'A backup is already in process'], 400);
        }

        try {
            $data = $manageProcess->init($params, $version);
			return $this->returnResponse(['code' => CodeResponse::SUCCESS, 'data' => $data, 'message' => 'Backup scheduled']);
        } catch (BackupNotCreated $e) {
            return $this->returnResponse(['code' => CodeResponse::ERROR, 'message' => $e->getMessage()], 400);
        }

    }
}
