<?php
namespace WPUmbrella\Controller\Backup;

if (!defined('ABSPATH')) {
    exit;
}

use WPUmbrella\Core\Models\AbstractController;
use WPUmbrella\Services\Backup\V2\BackupManageProcess;
use ActionScheduler;
use ActionScheduler_QueueRunner;
use ActionScheduler_Store;
use ActionScheduler_Action;

class RunProcess extends AbstractController
{

	const HOOK = 'wp_umbrella_create_additional_runner';
    const ACTION = 'wp_umbrella_create_additional_runner_action';
    const NONCE = 'wp_umbrella_create_additional_runner_nonce';

    public function executePost($params)
    {
		if(!defined('WP_UMBRELLA_INIT_BACKUP')){
			define('WP_UMBRELLA_INIT_BACKUP', true);
		}

		$manageProcess = wp_umbrella_get_service('BackupManageProcess');
        $isRunning = $manageProcess->isBackupInProgress();

		if(!$isRunning){
			return $this->returnResponse([
				'code' => 'no_process',
			]);
		}


		if(!function_exists('as_get_scheduled_actions')){
			return $this->returnResponse([
				'code' => 'no_action_scheduler',
			]);
		}

		$args = [
            'group' => BackupManageProcess::GROUP,
            'per_page' => 1,
            'status' => ActionScheduler_Store::STATUS_PENDING,
            'claimed' => false
        ];


        $actionsIds = as_get_scheduled_actions($args, 'ids');

        if (empty($actionsIds)) {
			return $this->returnResponse([
				'code' => 'in_progress',
			]);
        }

		try {
			$actionId = current($actionsIds);

			$runner = new ActionScheduler_QueueRunner();
			$runner->process_action( $actionId, 'WP Umbrella Backup' );
		} catch (\Exception $e) {
			return $this->returnResponse([
				'code' => 'error',
				'message' => $e->getMessage(),
			]);
		}

        return $this->returnResponse([
            'code' => 'success'
        ]);


    }
}
