<?php
namespace WPUmbrella\Controller\Restore;

if (!defined('ABSPATH')) {
    exit;
}

use WPUmbrella\Core\Models\AbstractController;
use WPUmbrella\Core\Restore\Memento\RestoreOriginator;
use WPUmbrella\Core\Restore\Builder\RestoreBuilder;

class Scan extends AbstractController
{
    public function executeGet($params)
    {
		if(!isset($params['restorationId'])){
			return $this->returnResponse(['code' => 'missing_parameters', 'message' => 'Restoration id is missing'], 400);
		}

		if(wp_umbrella_get_service('RestoreRunner')->restoreInProgress()){
			$originator = wp_umbrella_get_service('RestoreConfiguration')->getOriginatorFromFile();
			$restorationId = $originator->getValueInState('restoration_id');
			if($restorationId !== null && $restorationId != $params['restorationId']){
				return $this->returnResponse(['code' => 'restoration_in_progress', 'message' => 'Restore in progress'], 400);
			}
		}

        $director = wp_umbrella_get_service('RestoreDirector');

        $builder = new RestoreBuilder();
        $director->setBuilder($builder);

        $director->buildScanRestoration();
        $kernel = $builder->getKernel();

        $originator = new RestoreOriginator();

		$originator->setValueInState('restoration_id', $params['restorationId']);
        $kernel->execute($originator);

        if ($kernel->hasError()) {
            return $this->returnResponse($kernel->getError(), 400);
        }

        return $this->returnResponse(['code' => 'success'], 200);
    }
}
