<?php
namespace WPUmbrella\Controller\Restore;

if (!defined('ABSPATH')) {
    exit;
}

use WPUmbrella\Core\Models\AbstractController;
use WPUmbrella\Core\Restore\Memento\RestoreOriginator;
use WPUmbrella\Core\Restore\Builder\RestoreBuilder;

class ScanDownload extends AbstractController
{
    public function executeGet($params)
    {
        if (!isset($params['zip_size_bytes_file']) && !isset($params['zip_size_bytes_database'])) {
            return $this->returnResponse(['code' => 'missing_parameters', 'message' => 'Zip size bytes is required'], 400);
        }

        $director = wp_umbrella_get_service('RestoreDirector');

        $builder = new RestoreBuilder();
        $director->setBuilder($builder);

        $director->buildScanDownloadRestoration();
        $kernel = $builder->getKernel();

        $originator = wp_umbrella_get_service('RestoreConfiguration')->getOriginatorFromFile();

        $originator->setValueInState('zip_size_bytes_file', isset($params['zip_size_bytes_file']) ? $params['zip_size_bytes_file'] : null);
        $originator->setValueInState('zip_size_bytes_database', isset($params['zip_size_bytes_database']) ? $params['zip_size_bytes_database'] : null);

        $kernel->execute($originator);

        if ($kernel->hasError()) {
            return $this->returnResponse($kernel->getError(), 400);
        }

        return $this->returnResponse(['code' => 'success'], 200);
    }
}
