<?php
namespace WPUmbrella\Core;

use WPUmbrella\Core\Container\ContainerSkypress;
use WPUmbrella\Core\Container\ManageContainer;
use WPUmbrella\Core\Hooks\ActivationHook;
use WPUmbrella\Core\Hooks\DeactivationHook;
use WPUmbrella\Core\Hooks\ExecuteHooks;
use WPUmbrella\Core\Hooks\ExecuteHooksBackend;
use WPUmbrella\Core\Hooks\ExecuteHooksFrontend;
use WPUmbrella\Helpers\Controller;

abstract class Kernel
{
    protected static $container = null;


    protected static $data = ['slug' => null, 'main_file' => null, 'file' => null, 'root' => ''];

    public static function setSettings($data)
    {
        self::$data = array_merge(self::$data, $data);
    }

    public static function setContainer(ManageContainer $container)
    {
        self::$container = self::getDefaultContainer();
    }

    protected static function getDefaultContainer()
    {
        return new ContainerSkypress();
    }

    public static function getContainer()
    {
        if (null === self::$container) {
            self::$container = self::getDefaultContainer();
        }

        return self::$container;
    }

    public static function handleHooksPlugin()
    {
        require_once WP_UMBRELLA_DIR . '/src/Async/ActionSchedulerSendErrors.php';
        require_once WP_UMBRELLA_DIR . '/src/Async/ActionSchedulerSnapshotData.php';

        switch (current_filter()) {
            case 'plugins_loaded':

                load_plugin_textdomain('wp-health', false, WP_UMBRELLA_LANGUAGES);

                foreach (self::getContainer()->getActions() as $key => $class) {
                    if (!class_exists($class)) {
                        continue;
                    }

                    $class = new $class();

                    switch (true) {
                        case $class instanceof ExecuteHooksBackend:
                            if (is_admin()) {
                                $class->hooks();
                            }
                            break;

                        case $class instanceof ExecuteHooksFrontend:
                            if (!is_admin()) {
                                $class->hooks();
                            }
                            break;

                        case $class instanceof ExecuteHooks:
                            $class->hooks();
                            break;
                    }
                }
                break;
            case 'activate_' . self::$data['slug'] . '/' . self::$data['slug'] . '.php':
                foreach (self::getContainer()->getActions() as $key => $class) {
                    if (!class_exists($class)) {
                        continue;
                    }

                    $class = new $class();
                    if ($class instanceof ActivationHook) {
                        $class->activate();
                    }
                }
                break;
            case 'deactivate_' . self::$data['slug'] . '/' . self::$data['slug'] . '.php':
                foreach (self::getContainer()->getActions() as $key => $class) {
                    if (!class_exists($class)) {
                        continue;
                    }

                    $class = new $class();
                    if ($class instanceof DeactivationHook) {
                        $class->deactivate();
                    }
                }
                break;
        }
    }

    /**
     * @static
     *
     * @param string $path
     * @param string $type
     * @param string $namespace
     *
     * @return void
     */
    public static function buildClasses($path, $type, $namespace = '')
    {
        try {
            $files = array_diff(scandir($path), ['..', '.']);
            foreach ($files as $filename) {
                $pathCheck = $path . '/' . $filename;

                if (is_dir($pathCheck)) {
                    self::buildClasses($pathCheck, $type, $namespace . $filename . '\\');
                    continue;
                }

                $pathinfo = pathinfo($filename);
                if (isset($pathinfo['extension']) && 'php' !== $pathinfo['extension']) {
                    continue;
                }

                $data = '\\WPUmbrella\\' . $namespace . str_replace('.php', '', $filename);

                switch ($type) {
                    case 'services':
                        self::getContainer()->setService($data);
                        break;
                    case 'actions':
                        self::getContainer()->setAction($data);
                        break;
                }
            }
        } catch (\Exception $e) {
        }
    }

    /**
     * Build Container.
     */
    public static function buildContainer()
    {
        self::buildClasses(self::$data['root'] . '/src/Services', 'services', 'Services\\');
        self::buildClasses(self::$data['root'] . '/src/Actions', 'actions', 'Actions\\');
    }

    public static function getControllers()
    {
        return [
            '/v1/white-label' => [
                'route' => '/white-label',
                'methods' => [
                    [
                        'method' => 'POST',
                        'class' => \WPUmbrella\Controller\Options\WhiteLabel::class,
                        'options' => [
                            'prevent_active' => true,
                            'permission' => Controller::PERMISSION_AUTHORIZE,
                        ]
                    ],
                ]
            ],
            '/v1/issues-monitoring' => [
                'route' => '/issues-monitoring',
                'methods' => [
                    [
                        'method' => 'POST',
                        'class' => \WPUmbrella\Controller\Options\IssuesMonitoring::class,
                        'options' => [
                            'prevent_active' => true,
                            'permission' => Controller::PERMISSION_AUTHORIZE,
                        ]
                    ],
                ]
            ],
            '/v1/plugins' => [
                'route' => '/plugins',
                'methods' => [
                    [
                        'method' => 'GET',
                        'class' => \WPUmbrella\Controller\Plugin\Data::class,
                        'options' => [
                            'prevent_active' => true,
                            'permission' => Controller::PERMISSION_AUTHORIZE,
                        ]
                    ],
                ]
            ],
            '/v1/plugins/single' => [
                'route' => '/plugins/single',
                'methods' => [
                    [
                        'method' => 'GET',
                        'class' => \WPUmbrella\Controller\Plugin\DataSingle::class,
                        'options' => [
                            'prevent_active' => true,
                            'permission' => Controller::PERMISSION_AUTHORIZE,
                        ]
                    ],
                ]
            ],
            '/v1/update-plugins' => [
                'route' => '/update-plugins',
				'hook' => 'wp_loaded',
				'priority' => MAX_PRIORITY_HOOK,
                'methods' => [
                    [
                        'method' => 'POST',
                        'class' => \WPUmbrella\Controller\Plugin\UpdateMultiple::class,
                        'options' => [
                            'prevent_active' => true,
                            'permission' => Controller::PERMISSION_AUTHORIZE,
                        ]
                    ],
                ]
            ],
            '/v1/update-plugin' => [
                'route' => '/update-plugin',
				'hook' => 'wp_loaded',
				'priority' => MAX_PRIORITY_HOOK,
                'methods' => [
                    [
                        'method' => 'POST',
                        'class' => \WPUmbrella\Controller\Plugin\Update::class,
                        'options' => [
                            'prevent_active' => true,
                            'permission' => Controller::PERMISSION_AUTHORIZE,
                        ]
                    ],
                ]
            ],
            '/v1/delete-plugin' => [
                'route' => '/plugin-rollback',
                'methods' => [
                    [
                        'method' => 'DELETE',
                        'class' => \WPUmbrella\Controller\Plugin\Delete::class,
                        'options' => [
                            'prevent_active' => true,
                            'permission' => Controller::PERMISSION_AUTHORIZE,
                        ]
                    ],
                ]
            ],
            '/v1/install-plugin' => [
                'route' => '/install-plugin',
                'methods' => [
                    [
                        'method' => 'POST',
                        'class' => \WPUmbrella\Controller\Plugin\Delete::class,
                        'options' => [
                            'prevent_active' => true,
                            'permission' => Controller::PERMISSION_AUTHORIZE,
                        ]
                    ],
                ]
            ],
            '/v1/activate-plugin' => [
                'route' => '/activate-plugin',
                'methods' => [
                    [
                        'method' => 'POST',
                        'class' => \WPUmbrella\Controller\Plugin\Activate::class,
                        'options' => [
                            'prevent_active' => false,
                            'permission' => Controller::PERMISSION_AUTHORIZE,
                        ]
                    ],
                ]
            ],
            '/v1/deactivate-plugin' => [
                'route' => '/deactivate-plugin',
                'methods' => [
                    [
                        'method' => 'POST',
                        'class' => \WPUmbrella\Controller\Plugin\Deactivate::class,
                        'options' => [
                            'prevent_active' => true,
                            'permission' => Controller::PERMISSION_AUTHORIZE,
                        ]
                    ],
                ]
            ],
            '/v1/rollback-plugin' => [
                'route' => '/rollback-plugin',
                'methods' => [
                    [
                        'method' => 'POST',
                        'class' => \WPUmbrella\Controller\Plugin\Rollback::class,
                        'options' => [
                            'prevent_active' => true,
                            'permission' => Controller::PERMISSION_AUTHORIZE,
                        ]
                    ],
                ]
            ],
            '/v1/themes' => [
                'route' => '/themes',
                'methods' => [
                    [
                        'method' => 'GET',
                        'class' => \WPUmbrella\Controller\Theme\Data::class,
                        'options' => [
                            'prevent_active' => true,
                            'permission' => Controller::PERMISSION_AUTHORIZE,
                        ]
                    ],
                ]
            ],
            '/v1/snapshot' => [
                'route' => '/snapshot',
                'methods' => [
                    [
                        'method' => 'GET',
                        'class' => \WPUmbrella\Controller\Snapshot::class,
                        'options' => [
                            'prevent_active' => true,
                            'permission' => Controller::PERMISSION_AUTHORIZE,
                        ]
                    ],
                ]
            ],
            '/v1/update-theme' => [
                'route' => '/update-theme',
                'methods' => [
                    [
                        'method' => 'POST',
                        'class' => \WPUmbrella\Controller\Theme\Update::class,
                        'options' => [
                            'prevent_active' => true,
                            'permission' => Controller::PERMISSION_AUTHORIZE,
                        ]
                    ],
                ]
            ],
            '/v1/informations' => [
                'route' => '/informations',
                'methods' => [
                    [
                        'method' => 'GET',
                        'class' => \WPUmbrella\Controller\UmbrellaInformations::class,
                        'options' => [
                            'prevent_active' => true,
                            'permission' => Controller::PERMISSION_AUTHORIZE,
                        ]
                    ],
                ]
            ],
            '/v1/directories' => [
                'route' => '/directories',
                'methods' => [
                    [
                        'method' => 'GET',
                        'class' => \WPUmbrella\Controller\Directories::class,
                        'options' => [
                            'prevent_active' => true,
                            'permission' => Controller::PERMISSION_AUTHORIZE,
                        ]
                    ],
                ]
            ],
            '/v1/tables' => [
                'route' => '/tables',
                'methods' => [
                    [
                        'method' => 'GET',
                        'class' => \WPUmbrella\Controller\Tables::class,
                        'options' => [
                            'prevent_active' => true,
                            'permission' => Controller::PERMISSION_AUTHORIZE,
                        ]
                    ],
                ]
            ],
            '/v1/wordpress-data' => [
                'route' => '/wordpress-data',
                'methods' => [
                    [
                        'method' => 'GET',
                        'class' => \WPUmbrella\Controller\WordPressInfo::class,
                        'options' => [
                            'prevent_active' => true,
                            'permission' => Controller::PERMISSION_AUTHORIZE,
                        ]
                    ],
                ]
            ],
            '/v1/wordpress-sizes' => [
                'route' => '/wordpress-sizes',
                'methods' => [
                    [
                        'method' => 'GET',
                        'class' => \WPUmbrella\Controller\WordPressSize::class,
                        'options' => [
                            'prevent_active' => true,
                            'permission' => Controller::PERMISSION_AUTHORIZE,
                        ]
                    ],
                ]
            ],
            '/v1/users' => [
                'route' => '/users',
                'methods' => [
                    [
                        'method' => 'GET',
                        'class' => \WPUmbrella\Controller\User\Data::class,
                        'options' => [
                            'prevent_active' => true,
                            'permission' => Controller::PERMISSION_AUTHORIZE,
                        ]
                    ],
                ]
            ],
            '/v1/backups' => [
                'route' => '/backups',
				'version' => 'v1',
                'methods' => [
                    [
                        'method' => 'POST',
                        'class' => \WPUmbrella\Controller\Backup\Init::class,
                        'options' => [
                            'prevent_active' => true,
                            'permission' => Controller::PERMISSION_AUTHORIZE,
                        ]
                    ],
                    [
                        'method' => 'GET',
                        'class' => \WPUmbrella\Controller\Backup\CurrentProcess::class,
                        'options' => [
                            'prevent_active' => true,
                            'permission' => Controller::PERMISSION_AUTHORIZE
                        ]
                    ],
                    [
                        'method' => 'DELETE',
                        'class' => \WPUmbrella\Controller\Backup\DeleteProcess::class,
                        'options' => [
                            'prevent_active' => true,
                            'permission' => Controller::PERMISSION_AUTHORIZE
                        ]
                    ]
                ]
            ],
            '/v1/backups/run' => [
                'route' => '/backups/run',
				'version' => 'v1',
                'methods' => [
                    [
                        'method' => 'POST',
                        'class' => \WPUmbrella\Controller\Backup\RunProcess::class,
                        'options' => [
                            'prevent_active' => true,
                            'permission' => Controller::PERMISSION_AUTHORIZE,
                        ]
                    ],
                ]
            ],
            '/v1/backups/prepare-batch-database' => [
                'route' => '/backups/prepare-batch-database',
                'methods' => [
                    [
                        'method' => 'POST',
                        'class' => \WPUmbrella\Controller\Backup\PrepareBatchDatabase::class,
                        'options' => [
                            'prevent_active' => true,
                            'permission' => Controller::PERMISSION_AUTHORIZE
                        ]
                    ],
                ]
            ],
            '/v1/backups/check-batch-database' => [
                'route' => '/backups/check-batch-database',
                'methods' => [
                    [
                        'method' => 'POST',
                        'class' => \WPUmbrella\Controller\Backup\CheckBatchDatabase::class,
                        'options' => [
                            'prevent_active' => true,
                            'permission' => Controller::PERMISSION_AUTHORIZE
                        ]
                    ],
                ]
            ],
            '/v1/backups/scan' => [
                'route' => '/backups/scan',
                'methods' => [

                    [
                        'method' => 'GET',
                        'class' => \WPUmbrella\Controller\Backup\Scan::class,
                        'options' => [
                            'prevent_active' => true,
                            'permission' => Controller::PERMISSION_AUTHORIZE
                        ]
                    ],

                ]
            ],

            '/v1/clear-cache' => [
                'route' => '/clear-cache',
                'methods' => [
                    [
                        'method' => 'POST',
                        'class' => \WPUmbrella\Controller\ClearCache::class,
                        'options' => [
                            'prevent_active' => true,
                            'permission' => Controller::PERMISSION_AUTHORIZE,
                        ]
                    ],
                ]
            ],

            '/v1/cores' => [
                'route' => '/cores',
                'methods' => [
                    [
                        'method' => 'GET',
                        'class' => \WPUmbrella\Controller\Core\Data::class,
                        'options' => [
                            'prevent_active' => true,
                            'permission' => Controller::PERMISSION_AUTHORIZE,
                        ]
                    ],
                ]
            ],
            '/v1/update-core' => [
                'route' => '/update-core',
                'methods' => [
                    [
                        'method' => 'POST',
                        'class' => \WPUmbrella\Controller\Core\Update::class,
                        'options' => [
                            'prevent_active' => true,
                            'permission' => Controller::PERMISSION_AUTHORIZE,
                        ]
                    ],
                ]
            ],
            '/v1/restores' => [
                'route' => '/restores',
                'methods' => [
                    [
                        'method' => 'DELETE',
                        'class' => \WPUmbrella\Controller\Restore\DeleteProcess::class,
                        'options' => [
                            'prevent_active' => true,
                            'permission' => Controller::PERMISSION_AUTHORIZE
                        ]
                    ]
                ]
            ],
            '/v1/restores/scan' => [
                'route' => '/restores/scan',
                'methods' => [
                    [
                        'method' => 'GET',
                        'class' => \WPUmbrella\Controller\Restore\Scan::class,
                        'options' => [
                            'prevent_active' => true,
                            'permission' => Controller::PERMISSION_AUTHORIZE
                        ]
                    ]
                ]
            ],
            '/v1/restores/check' => [
                'route' => '/restores/check',
                'methods' => [
                    [
                        'method' => 'GET',
                        'class' => \WPUmbrella\Controller\Restore\Check::class,
                        'options' => [
                            'prevent_active' => true,
                            'permission' => Controller::PERMISSION_AUTHORIZE
                        ]
                    ]
                ]
            ],
            '/v1/restores/download-restore-plugin' => [
                'route' => '/restores/download-restore-plugin',
                'methods' => [
                    [
                        'method' => 'POST',
                        'class' => \WPUmbrella\Controller\Restore\DownloadRestorePlugin::class,
                        'options' => [
                            'prevent_active' => true,
                            'permission' => Controller::PERMISSION_AUTHORIZE
                        ]
                    ]
                ]
            ],
            '/v1/restores/scan-download' => [
                'route' => '/restores/scan-download',
                'methods' => [
                    [
                        'method' => 'GET',
                        'class' => \WPUmbrella\Controller\Restore\ScanDownload::class,
                        'options' => [
                            'prevent_active' => true,
                            'permission' => Controller::PERMISSION_AUTHORIZE
                        ]
                    ]
                ]
            ],
            '/v1/restores/download' => [
                'route' => '/restores/download',
                'methods' => [
                    [
                        'method' => 'POST',
                        'class' => \WPUmbrella\Controller\Restore\ZipDownload::class,
                        'options' => [
                            'prevent_active' => true,
                            'permission' => Controller::PERMISSION_AUTHORIZE,
                        ]
                    ]
                ]
            ],
            '/v1/restores/prepare-files' => [
                'route' => '/restores/prepare-files',
                'methods' => [
                    [
                        'method' => 'POST',
                        'class' => \WPUmbrella\Controller\Restore\RestorePrepareFiles::class,
                        'options' => [
                            'prevent_active' => true,
                            'permission' => Controller::PERMISSION_AUTHORIZE,
                        ]
                    ]
                ]
            ],
            '/v1/restores/files' => [
                'route' => '/restores/files',
                'methods' => [
                    [
                        'method' => 'POST',
                        'class' => \WPUmbrella\Controller\Restore\RestoreFiles::class,
                        'options' => [
                            'prevent_active' => true,
                            'permission' => Controller::PERMISSION_AUTHORIZE,
                        ]
                    ]
                ]
            ],
            '/v1/restores/database' => [
                'route' => '/restores/database',
                'methods' => [
                    [
                        'method' => 'POST',
                        'class' => \WPUmbrella\Controller\Restore\RestoreDatabase::class,
                        'options' => [
                            'prevent_active' => true,
                            'permission' => Controller::PERMISSION_AUTHORIZE,
                        ]
                    ]
                ]
            ],
            '/v1/maintenance' => [
                'route' => '/maintenance',
                'methods' => [
                    [
                        'method' => 'POST',
                        'class' => \WPUmbrella\Controller\MaintenanceMode::class,
                        'options' => [
                            'prevent_active' => true,
                            'permission' => Controller::PERMISSION_AUTHORIZE,
                        ]
						],
                    [
                        'method' => 'DELETE',
                        'class' => \WPUmbrella\Controller\MaintenanceMode::class,
                        'options' => [
                            'prevent_active' => true,
                            'permission' => Controller::PERMISSION_AUTHORIZE,
                        ]
                    ]
                ]
            ],
            '/v1/languages' => [
                'route' => ''
            ],
            '/v1/options' => [
                'route' => ''
            ],
            '/v1/login' => [
                'route' => '/login',
				'methods' => [
					[
                        'method' => 'GET',
                        'class' => \WPUmbrella\Controller\Login::class,
                        'options' => [
                            'prevent_active' => true,
                        ]
                    ]
				]
            ],
            '/v1/umbrella-nonce-login' => [
                'route' => '/umbrella-nonce-login',
				'methods' => [
					[
                        'method' => 'POST',
                        'class' => \WPUmbrella\Controller\UmbrellaNonceLogin::class,
                        'options' => [
                            'prevent_active' => true,
                            'permission' => Controller::PERMISSION_AUTHORIZE,
                        ]
                    ]
				]
            ],
            '/v1/logs' => [
				'route' => '/logs',
				'methods' => [
					[
                        'method' => 'GET',
                        'class' => \WPUmbrella\Controller\Logs::class,
                        'options' => [
                            'prevent_active' => true,
                            'permission' => Controller::PERMISSION_AUTHORIZE,
                        ]
                    ]
				]
            ],
        ];
    }

    public static function canExecute()
    {
        $headers = \wp_umbrella_get_headers();

        $token = isset($headers['x-umbrella']) ? $headers['x-umbrella'] : null;
        $projectId = isset($headers['x-project']) ? $headers['x-project'] : null;

        $response = wp_umbrella_get_service('ApiWordPressPermission')->isAuthorizedToken($token);

        if (!isset($response['authorized']) || !$response['authorized']) {
            return false;
        }

        $response = wp_umbrella_get_service('ApiWordPressPermission')->isAuthorizedProjectId($projectId);

        if (!isset($response['authorized']) || !$response['authorized']) {
            return false;
        }

        return true;
    }

	public static function getWPEngineCookieValue() {
        return hash("sha256", 'wpe_auth_salty_dog|'.WPE_APIKEY);
    }


	/**
	 * In WP Engine hosting only requests from logged in users with auth cookies are given filesystem
	 *  write access.
	 */
	public static function setCookies($user) {

		$user_id = $user->ID;

		$cookies = [];
		$secure  = is_ssl();
		$secure  = apply_filters( 'secure_auth_cookie', $secure, $user_id );

		if ( $secure ) {
			$auth_cookie_name = SECURE_AUTH_COOKIE;
			$scheme           = 'secure_auth';
		} else {
			$auth_cookie_name = AUTH_COOKIE;
			$scheme           = 'auth';
		}

		$expiration = time() + ( DAY_IN_SECONDS * 14 );

		$cookies[ $auth_cookie_name ] = wp_generate_auth_cookie( $user_id, $expiration, $scheme );
		$cookies[ LOGGED_IN_COOKIE ]  = wp_generate_auth_cookie( $user_id, $expiration, 'logged_in' );
		if (defined('WPE_APIKEY')) {
            $cookieValue = self::getWPEngineCookieValue();
            setcookie('wpe-auth', $cookieValue, 0, '/', '', force_ssl_admin(), true);
        }

	}

    public static function trySetupAdmin()
    {
		if(!defined('WP_UMBRELLA_IS_ADMIN')){
			define('WP_UMBRELLA_IS_ADMIN', true);
		}

        if (!defined('WP_ADMIN')) {
            define('WP_ADMIN', true);
        }

        if (!defined('WP_NETWORK_ADMIN')) {
            define('WP_NETWORK_ADMIN', false);
        }

        if (isset($_GET['import']) && !defined('WP_LOAD_IMPORTERS')) {
            define('WP_LOAD_IMPORTERS', true);
        }


        if (!function_exists('get_current_screen')) {
            include_once ABSPATH . '/wp-admin/includes/class-wp-screen.php';
            include_once ABSPATH . '/wp-admin/includes/screen.php';
        }

		try {
			$GLOBALS['hook_suffix'] = '';
			\WP_Screen::get('')->set_current_screen();
		} catch (\Exception $e) {
			// No black magic
		}

        if (!function_exists('wp_set_current_user')) {
            include_once ABSPATH . '/wp-includes/pluggable.php';
        }


        $user = wp_umbrella_get_service('UsersProvider')->getUserAdminCanBy();
        if (!$user) {
            return false;
        }

		// Authenticated user
		wp_cookie_constants();
		wp_set_current_user($user->ID, $user->user_login);
		self::setCookies($user);
    }

    protected static function buildActionScheduler(){

        if(function_exists('action_scheduler_initialize_3_dot_3_dot_0')){
            action_scheduler_initialize_3_dot_3_dot_0();
            \ActionScheduler_Versions::initialize_latest_version();
        }

        if ( ! class_exists( 'ActionScheduler_Store' ) ) {
            require_once WP_UMBRELLA_DIR . '/thirds/action-scheduler/deprecated/ActionScheduler_Store_Deprecated.php';
            require_once WP_UMBRELLA_DIR . '/thirds/action-scheduler/classes/abstracts/ActionScheduler_Store.php';
            require_once WP_UMBRELLA_DIR . '/thirds/action-scheduler/classes/abstracts/ActionScheduler_Abstract_Schema.php';
            require_once WP_UMBRELLA_DIR . '/thirds/action-scheduler/classes/schema/ActionScheduler_StoreSchema.php';
        }

        $tables = [
            \ActionScheduler_StoreSchema::ACTIONS_TABLE,
            \ActionScheduler_StoreSchema::CLAIMS_TABLE,
            \ActionScheduler_StoreSchema::GROUPS_TABLE,
        ];

        global $wpdb;
        foreach ( $tables as $table ) {
            $wpdb->tables[] = $table;
            $name           = $GLOBALS[ 'wpdb' ]->prefix . $table;
            $wpdb->$table   = $name;
        }
    }


	public static function setPluginPriority()
	{
		$pluginBasename = 'wp-health/wp-health.php';
		$activePlugins  = get_option('active_plugins');

		if (!is_array($activePlugins) || reset($activePlugins) === $pluginBasename) {
			return;
		}

		$key = array_search($pluginBasename, $activePlugins);

		if ($key === false || $key === null) {
			return;
		}

		unset($activePlugins[$key]);
		array_unshift($activePlugins, $pluginBasename);
		update_option('active_plugins', array_values($activePlugins));
	}


    public static function execute($data)
    {
        if (!class_exists('ActionScheduler')) {
            require_once WP_UMBRELLA_DIR . '/thirds/action-scheduler/action-scheduler.php';
        }

        self::setSettings($data);
        self::buildContainer();

        $headers = \wp_umbrella_get_headers();

		$universalProcess = [
			'load' => false,
			'hook' => 'wp',
			'priority' => 10,
			'action' => null
		];

        if (
            (isset($headers['x-action']) && isset($headers['x-umbrella']) && isset($headers['x-project'])) ||
            (isset($_GET['x-action']) && isset($_GET['x-umbrella']) && isset($_GET['x-project']))
        ) {
            $controllers = self::getControllers();
            $action = isset($headers['x-action']) ? $headers['x-action'] : $_GET['x-action'];

            if (self::canExecute() && array_key_exists($action, $controllers)) {
				self::setPluginPriority();
				$item = $controllers[$action];
				$universalProcess = [
					'load' => true,
					'action' => $action,
					'hook' => isset($item['hook']) ? $item['hook'] : 'wp',
					'priority' => isset($item['priority']) ? $item['priority'] : 10,
				];

            }
        }

		$delayForPlugins = [];
		if(defined('WP_PLUGIN_DIR')){
			$delayForPlugins = [
				WP_PLUGIN_DIR . '/wp-marketing-automations/wp-marketing-automations.php'
			];
		}

		$needDelayHook = false;
		foreach($delayForPlugins as $plugin){
			if(file_exists($plugin)){
				$needDelayHook = true;
				break;
			}
		}

		if($universalProcess['load']){

			self::trySetupAdmin();
			self::buildActionScheduler();

			if($universalProcess['action'] === '/v1/snapshot' && !$needDelayHook){
				wp_umbrella_get_service('RequestSettings')->buildRequestSettings();
			}


			add_action($universalProcess['hook'], function() use ($universalProcess) {
				if($universalProcess['action'] !== '/v1/snapshot'){
					wp_umbrella_get_service('RequestSettings')->buildRequestSettings();
				}
				require_once WP_UMBRELLA_DIR . '/request/umbrella-application.php';
				return;
			}, $universalProcess['priority']);
		}



        add_action('plugins_loaded', [__CLASS__, 'handleHooksPlugin'], 10);
        register_activation_hook($data['file'], [__CLASS__, 'handleHooksPlugin']);
        register_deactivation_hook($data['file'], [__CLASS__, 'handleHooksPlugin']);
    }
}
