<?php
namespace WPUmbrella\Services\Backup;

if (!defined('ABSPATH')) {
    exit;
}

use WPUmbrella\Core\Backup\Memento\BackupOriginator;
use Symfony\Component\Finder\Finder;

class BackupConfiguration
{

	protected function getPath(){
		return sprintf('%s/%s', WP_UMBRELLA_DIR_SCRATCH_BACKUP, 'config.json');
	}

    public function getOriginatorFromFile()
    {
        $originator = new BackupOriginator();

        try {
			// No need for the moment
            // $path = $this->getPath();;
            // if (file_exists($path)) {
            //     $config = json_decode(file_get_contents($path), true);
            //     $originator->setState($config);
            // }

			$tables = $this->getTables();
			$originator->setValueInState('tables', $tables);

        } catch (\Exception $e) {
            return $originator;
        }


		return $originator;
    }

	public function setTables($tables){

		foreach($tables as $index =>  $table){
			$this->setTable($index, $table);
		}
	}

	public function setTable($index, $table){
		$path = sprintf('%s/%s-table.json', WP_UMBRELLA_DIR_SCRATCH_BACKUP, $table['table']);

		$tables = isset($state['tables']) ? $table['tables'] : [];

		$current = json_encode($table);
		file_put_contents($path, $current);
	}

	public function getTables(){

		if(!file_exists(WP_UMBRELLA_DIR_SCRATCH_BACKUP)){
			return [];
		}

        $finder = new Finder();
        $finder->files()
                ->in(WP_UMBRELLA_DIR_SCRATCH_BACKUP)
                ->ignoreDotFiles(true)
				->name('*-table.json');

		$tables = [];
		foreach ($finder as $file) {
			$content = json_decode($file->getContents(), true);
			$tables[] = $content;
		}

		return $tables;

	}

	public function removeTables(){
		if(!file_exists(WP_UMBRELLA_DIR_SCRATCH_BACKUP)){
			return;
		}

        $finder = new Finder();
        $finder->files()
                ->in(WP_UMBRELLA_DIR_SCRATCH_BACKUP)
                ->ignoreDotFiles(true)
				->name('*-table.json');

		try {
			foreach ($finder as $file) {
				unlink($file->getRealPath());
			}
		} catch (\Exception $e) {
			// Do nothing
		}
	}

	public function saveOriginatorToFile($originator){
		if(!file_exists(WP_UMBRELLA_DIR_SCRATCH_BACKUP)){
			@mkdir(WP_UMBRELLA_DIR_SCRATCH_BACKUP);
		}

		$path = $this->getPath();
		$state = $originator->getState();
		$tables = isset($state['tables']) ? $state['tables'] : [];

		if(!empty($tables)){
			$this->setTables($tables);
			unset($state['tables']);
		}

		// No need for the moment
		// $current = json_encode($state);
		// file_put_contents($path, $current);
	}
}


