<?php
namespace WPUmbrella\Services\Backup\QueueRunner;

if (!defined('ABSPATH')) {
    exit;
}

use WPUmbrella\Core\Backup\Builder\BackupBuilder;
use WPUmbrella\Models\Backup\BackupProcessedData;
use WPUmbrella\Core\Backup\Source\FinderBatchSource;
use WPUmbrella\Services\Backup\BackupBatchData;

class BackupQueueRunnerDatabase extends AbstractBackupQueue
{
    const TYPE = 'database';

    public function zipDatabase(BackupProcessedData $dataModel): BackupProcessedData
	{
        wp_umbrella_get_service('Logger')->info('ZIP database');

        $backupDirector = wp_umbrella_get_service('BackupDirector');
        $builder = new BackupBuilder();

        $profile = $backupDirector->constructBackupProfileProcessor($builder, $dataModel, self::TYPE);

        $backupExecutor = wp_umbrella_get_service('BackupExecutor');
        $backupExecutor->zip($profile);

        /**
         * @var BackupBatchData
         */
        $batchData = wp_umbrella_get_service('BackupBatchData');
        $batchData->setDataOption($dataModel->getData(), self::TYPE);

        return $dataModel;
    }

	protected function finishBackupDatabase($dataModel){
		$dataModel = $this->zipDatabase($dataModel);

		$dataModel->setTimestampEndDate(time());
		wp_umbrella_get_service('BackupBatchData')->setDataOption($dataModel->getData(), self::TYPE);

		// Send to destinations
		$dataModel = $this->sendToDestinations($dataModel, self::TYPE);
		$this->finishBackupAndSaveDataModel($dataModel, self::TYPE);
		wp_umbrella_get_service('BackupConfiguration')->removeTables();
	}

    public function run()
    {
        $this->setupProcess();


		$originator = wp_umbrella_get_service('BackupConfiguration')->getOriginatorFromFile();
		$tables = $originator->getValueInState('tables');
		if($tables === null || empty($tables)){
			$tables = wp_umbrella_get_service('BackupDatabaseConfiguration')->getTablesBatch();
			$originator->setValueInState('tables', $tables);

			wp_umbrella_get_service('BackupConfiguration')->saveOriginatorToFile($originator);
		}

        /**
         * @var BackupBatchData
         */
        $batchData = wp_umbrella_get_service('BackupBatchData');
        $dataModel = $batchData->getData(self::TYPE);

        $backupDirector = wp_umbrella_get_service('BackupDirector');
        $builder = new BackupBuilder();

        $profile = $backupDirector->constructBackupProfileOnlySQL($builder, $dataModel);
        $dataModel->createDefaultName('', true);

        $backupExecutor = wp_umbrella_get_service('BackupExecutor');

        $result = $backupExecutor->backupSources($profile);
        $response = $result[0];
        $newPosition = $response['iterator_position'];
        $dataModel = $response['processed_data'];

        if (!$response['success']) {
            $batchData->setDataOption($dataModel->getData(), self::TYPE);
            $this->saveCurrentDataModel($dataModel, self::TYPE);
            wp_umbrella_get_service('Logger')->error('Backup database failed - Position: ' . $response['iterator_position']);
            return;
        }

        wp_umbrella_get_service('Logger')->info('Iterator after process: ' . $newPosition);

        if ($newPosition >= count($tables)) { // Finish backup database
            $this->finishBackupDatabase($dataModel);
            return;
        }

		// Prevent table not exist
		$current = $dataModel->getCurrentBatchProcessorSql();
		$currentTable = $originator->getTableByIterator($current);


		if($currentTable === null && count($tables) < $current+1) {
			//finish
			$this->finishBackupDatabase($dataModel);
			return;
		}
		else if($currentTable === null){
			$dataModel->setIteratorPositionSql($current + 1);
		}

		if (isset($currentTable['batch']) || isset($currentTable['batch_by_line'])) {
			$keyTableBatch = isset($currentTable['batch_by_line']) ? 'batch_by_line' : 'batch';
			$part = $dataModel->getPartBatchSqlProcessor();
			if(!isset($currentTable[$keyTableBatch][$part])) {
				$dataModel->setIteratorPositionSql($current + 1); // Next table
				$dataModel->setPartBatchSqlProcessor(0); // Reinit part
			}
		}


        // Continue
        $batchData->setDataOption($dataModel->getData(), self::TYPE);
        $this->saveCurrentDataModel($dataModel, self::TYPE);

        wp_umbrella_get_service('Logger')->info('Prepare new batch : ' . $newPosition);

        wp_umbrella_get_service('BackupRunner')->scheduledBatchDatabase();
    }
}
