<?php
namespace WPUmbrella\Services\Backup\QueueRunner\V2;

use WPUmbrella\Core\Backup\Builder\V2\BackupBuilder;
use WPUmbrella\Services\Backup\V2\BackupDirector;
use WPUmbrella\Core\Constants\CodeResponse;

class BackupQueueCheckDatabaseBatch extends AbstractBackupQueue
{

	public function setTypeClient($typeClient = "guzzle"){
		$this->typeClient = $typeClient;

		return $this;
	}

    protected function getClient(){
        $client = new \GuzzleHttp\Client();

        return $client;
    }

	protected function callWithGuzzle($data, $options = []){
		try {
			@set_time_limit(0);
			$client = $this->getClient();

			$json = [
				'table' => $data->getTableByCurrentBatch()['name'],
				'part' => $data->getBatchPart('database'),
			];

			if(isset($options['divided_memory'])){
				$json['divided_memory'] = $options['divided_memory'];
			}

			$response = $client->post(SITE_URL, [
				'headers' => [
					'X-Project' => PROJECT_ID,
					'X-Umbrella' => TOKEN,
					'X-Action' => '/v1/backups/check-batch-database'
				],
				'json' => $json,
				'timeout' => 0
			]);

			$response = json_decode($response->getBody()->getContents(), true);

			if($response['code'] == CodeResponse::SUCCESS){
				return [
					'success' => true,
					'data' => $response['data']
				];
			}

			return [
				'success' => false,
				'data' => [
					'code' => $response['code']
				]
			];

		} catch (\Exception $e) {
			return [
				'success' => false,
				'data' => []
			];
		}
	}

	protected function callWithWordPress($data, $options = []){
		try {
			@set_time_limit(0);

			$projectId = wp_umbrella_get_option('project_id');
			$token = wp_umbrella_get_option('api_key');

			$json =  [
				'table' => $data->getTableByCurrentBatch()['name'],
				'part' => $data->getBatchPart('database'),
			];

			if(isset($options['divided_memory'])){
				$json['divided_memory'] = $options['divided_memory'];
			}

            $response = wp_remote_post(site_url(), [
                'headers' => [
					'X-Project' => $projectId,
					'X-Umbrella' => $token,
					'X-Action' => '/v1/backups/check-batch-database'
				],
                'body' => json_encode($json),
                'timeout' => 0,
            ]);

            $response = json_decode(wp_remote_retrieve_body($response), true);

            return [
				'success' => true,
				'data' => $response['data']
			];

		} catch (\Exception $e) {
			return [
				'success' => false,
				'data' => []
			];
		}
	}

    public function run($options = [])
    {

		$manageProcess = wp_umbrella_get_service('BackupManageProcess');
		$data = $manageProcess->getBackupData();

		if($this->typeClient === 'guzzle'){
			return $this->callWithGuzzle($data, $options);
		}
		else if($this->typeClient === 'wordpress'){
			return $this->callWithWordPress($data, $options);
		}

    }
}
