<?php
namespace WPUmbrella\Services\Backup\QueueRunner\V2;

use WPUmbrella\Core\Backup\Builder\V2\BackupBuilder;
use WPUmbrella\Services\Backup\V2\BackupDirector;

class BackupQueuePrepareDatabaseBatch extends AbstractBackupQueue
{

	public function setTypeClient($typeClient = "guzzle"){
		$this->typeClient = $typeClient;

		return $this;
	}

    protected function getClient(){
        $client = new \GuzzleHttp\Client();

        return $client;
    }

	protected function callWithGuzzle($data){
		try {
			@set_time_limit(0);
			$client = $this->getClient();

			$response = $client->post(SITE_URL, [
				'headers' => [
					'X-Project' => PROJECT_ID,
					'X-Umbrella' => TOKEN,
					'X-Action' => '/v1/backups/prepare-batch-database'
				],
				'json' => [
					'table' => $data->getTableByCurrentBatch()['name']
				],
				'timeout' => 0
			]);

			$batchs = json_decode($response->getBody()->getContents(), true);

			return [
				'success' => true,
				'data' => $batchs
			];

		} catch (\Exception $e) {
			return [
				'success' => false,
				'data' => []
			];
		}
	}

	protected function callWithWordPress($data){
		try {
			@set_time_limit(0);

			$projectId = wp_umbrella_get_option('project_id');
			$token = wp_umbrella_get_option('api_key');

            $response = wp_remote_post(site_url(), [
                'headers' => [
					'X-Project' => $projectId,
					'X-Umbrella' => $token,
					'X-Action' => '/v1/backups/prepare-batch-database'
				],
                'body' => json_encode([
					'table' => $data->getTableByCurrentBatch()['name']
				]),
                'timeout' => 0,
            ]);

            $batchs = json_decode(wp_remote_retrieve_body($response), true);

            return [
				'success' => true,
				'data' => $batchs
			];

		} catch (\Exception $e) {
			return [
				'success' => false,
				'data' => []
			];
		}
	}

    public function run()
    {

		$manageProcess = wp_umbrella_get_service('BackupManageProcess');
		$data = $manageProcess->getBackupData();

		if($this->typeClient === 'guzzle'){
			return $this->callWithGuzzle($data);
		}
		else if($this->typeClient === 'wordpress'){
			return $this->callWithWordPress($data);
		}

    }
}
