<?php
namespace WPUmbrella\Services\Backup\QueueRunner\V2;


use WPUmbrella\Core\Backup\Builder\V2\BackupBuilder;
use WPUmbrella\Services\Backup\V2\BackupDirector;
use WPUmbrella\Core\Constants\CodeResponse;
use WPUmbrella\Core\Constants\BackupData;

class BackupQueueRunnerDatabase extends AbstractBackupQueue
{
    const TYPE = 'database';

	const NAME_SERVICE =  'BackupQueueRunnerDatabaseV2';

    public function zipDatabase($data)	{

        $backupDirector = wp_umbrella_get_service('BackupDirectorV2');
        $builder = new BackupBuilder();

        $profile = $backupDirector->constructBackupProfileProcessor($builder, $data, self::TYPE);

        $backupExecutor = wp_umbrella_get_service('BackupExecutorV2');
        $backupExecutor->zip($profile);

    }

	protected function finishBackupDatabase($data){
		$this->zipDatabase($data);

		$data->setFinish(self::TYPE);
		$data->setTimestampEndDate(time());
		wp_umbrella_get_service('BackupManageProcess')->updateBackupData($data->getData());

		// Send to destinations
		$data = $this->sendToDestinations($data, self::TYPE);
		$this->finishBackupAndSaveDataModel($data, self::TYPE);
	}

	protected function checkBatchIfNeeded($data){
		$currentTable = $data->getTableByCurrentBatch();

		if(!$data->hasTableNeedBatchByName($currentTable['name'])){
			return false;
		}

		$batchs = $data->getTableBatchsByName($currentTable['name']);
		$part = $data->getBatchPart('database');

		if(!isset($batchs[$part])){
			return false;
		}

		if(!$batchs[$part]['need_check']){
			return false;
		}

		return true;

	}

    public function run()
    {

		$manageProcess = wp_umbrella_get_service('BackupManageProcess');

		$data = $manageProcess->getBackupData();
		$finish = $data->getFinish(self::TYPE);

		if($finish){
			return [
				'success' => true,
				'data' => [
					'code' => CodeResponse::BACKUP_NEED_CLEANUP
				]
			];
		}

        $name = $manageProcess->createDefaultName([
			'title' => $data->getTitle(),
			'suffix' => $data->getSuffix(),
			'database' => true,
			'part' => $data->getBatchPart(self::TYPE),
			'backupId' => $data->getBackupId()
		]);

		$data->setName($name, self::TYPE);

		$part = $data->getBatchPart(self::TYPE);
		$currentTable = $data->getTableByCurrentBatch();
		$tablesBatchs = $data->getTableBatchsByName($currentTable['name']);

		if($part === 0 && $currentTable['need_batch'] && $tablesBatchs === null){
			return [
				'success' => true,
				'data' => [
					'code' => CodeResponse::BACKUP_TABLE_NEED_BATCH
				]
			];
		}

		/**
		 * @var BackupDirector
		 */
        $backupDirector = wp_umbrella_get_service('BackupDirectorV2');
        $builder = new BackupBuilder();

        $currentIterator = $data->getBatchIterator(self::TYPE);

		$checkBatch = $this->checkBatchIfNeeded($data);
		if($checkBatch){
			return [
				'success' => true,
				'data' => [
					'code' => CodeResponse::BACKUP_TABLE_CHECK_BATCH
				]
			];
		}

        $profile = $backupDirector->constructBackupProfileOnlySQL($builder, $data);

		$backupExecutor = wp_umbrella_get_service('BackupExecutorV2');
        $result = $backupExecutor->backupSources($profile);

		$response = $result[0];

		if (!$response['success']) {
			$data = $data->getData();
			$data['type'] = self::TYPE;
			$this->api->postErrorBackup($data);
			return [
				'success' => false,
				'data' => [
					'code' => CodeResponse::BACKUP_ERROR
				]
			];
        }

		$newBatchIterator = $response['iterator_position'];
        $newPart = $response['part'];

		$data->setBatchIterator(self::TYPE, $newBatchIterator)
			 ->setBatchPart(self::TYPE, $newPart);

		$currentTableAfterProcess = $data->getTableByCurrentBatch();

		// finish
		if($currentTableAfterProcess === null){

			$manageProcess->updateBackupData($data->getData());

			$this->finishBackupDatabase($data);

			return [
				'success' => true,
				'data' => [
					'code' => CodeResponse::BACKUP_DATABASE_FINISH,
				]
			];
		}
		else{
			$this->saveCurrentDataModel($data, self::TYPE);
		}

		$manageProcess->updateBackupData($data->getData());

		return [
			'success' => true,
			'data' => [
				'current_table' => $currentTableAfterProcess['name'],
				'iterator_position' => $data->getBatchIterator(self::TYPE),
				'part' => $data->getBatchPart(self::TYPE),
				'code' => CodeResponse::BACKUP_NEXT_PART_DATABASE,
			]
		];
    }
}
