<?php
namespace WPUmbrella\Services;

if (!defined('ABSPATH')) {
    exit;
}

class HostResolver
{
    public function getCurrentHost()
    {
        try {
			if (isset($_SERVER['KINSTA_CACHE_ZONE'])) {
				return 'kinsta';
			}

			if ((defined('DB_HOST') && strpos(DB_HOST, '.wpserveur.net') !== false)) {
				return 'wp-server';
			}

			if (class_exists('FlywheelNginxCompat')) {
				return 'flywheel';
			}

			if ( isset( $_SERVER['cw_allowed_ip'] ) ) {
				return 'cloudways';
			}

			if ( wp_umbrella_get_service('WordPressContext')->getConstant( 'IS_PRESSABLE' ) ) {
				return 'pressable';
			}

			if ( getenv( 'SPINUPWP_CACHE_PATH' ) ) {
				return 'spinupwp';
			}

			if ((class_exists( 'WpeCommon' ) && function_exists( 'wpe_param' )) ) {
				return 'wpengine';
			}

			if ( wp_umbrella_get_service('WordPressContext')->hasConstant( 'O2SWITCH_VARNISH_PURGE_KEY' ) ) {
				return 'o2switch';
			}

			if ( wp_umbrella_get_service('WordPressContext')->getConstant( 'WPCOMSH_VERSION' ) ) {
				return 'wordpresscom';
			}

			if (
				wp_umbrella_get_service('WordPressContext')->getConstant( '\Savvii\CacheFlusherPlugin::NAME_FLUSH_NOW' )
				&&
				wp_umbrella_get_service('WordPressContext')->getConstant( '\Savvii\CacheFlusherPlugin::NAME_DOMAINFLUSH_NOW' )
			) {
				return 'savvii';
			}

			if ( $this->isDreampress() ) {
				return 'dreampress';
			}

			if ( isset( $_SERVER['X-LSCACHE'] ) ) {
				return 'litespeed';
			}

			if ( class_exists( '\WPaas\Plugin' ) ) {
				return 'godaddy';
			}

			return 'other';
		} catch (\Exception $e) {
			return 'other';
		}
    }


	protected function isDreampress() {
		if ( ! isset( $_SERVER['DH_USER'] ) ) {
			return false;
		}

		return 'wp_' === substr( sanitize_key( wp_unslash( $_SERVER['DH_USER'] ) ), 0, 3 );
	}
}
